#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (C) 2015-2020 Richard Hughes <richard@hughsie.com>
#
# SPDX-License-Identifier: LGPL-2.1+
#
# pylint: disable=protected-access

import os
import struct
import fnmatch
import zlib
import ntpath

from typing import Optional, List

from cabarchive.file import CabFile
from cabarchive.errors import CorruptionError, NotSupportedError


FMT_CFHEADER = "<4sxxxxIxxxxIxxxxBBHHHHH"
FMT_CFFOLDER = "<IHH"
FMT_CFFILE = "<IIHHHH"
FMT_CFDATA = "<IHH"


def _chunkify(arr: bytes, size: int) -> List[bytearray]:
    """ Split up a bytestream into chunks """
    arrs = []
    for i in range(0, len(arr), size):
        chunk = bytearray(arr[i : i + size])
        arrs.append(chunk)
    return arrs


def _checksum_compute(content: bytes, seed: int = 0) -> int:
    """ Compute the MS cabinet checksum """
    csum = seed
    chunks = _chunkify(content, 4)
    for chunk in chunks:
        if len(chunk) == 4:
            ul = chunk[0]
            ul |= chunk[1] << 8
            ul |= chunk[2] << 16
            ul |= chunk[3] << 24
        else:
            # WTF: I can only assume this is a typo from the original
            # author of the cabinet file specification
            if len(chunk) == 3:
                ul = (chunk[0] << 16) | (chunk[1] << 8) | chunk[2]
            elif len(chunk) == 2:
                ul = (chunk[0] << 8) | chunk[1]
            elif len(chunk) == 1:
                ul = chunk[0]
        csum ^= ul
    return csum


class CabArchive(dict):
    """An object representing a Microsoft Cab archive """

    def __init__(self, buf: Optional[bytes] = None, flattern: bool = False):
        """ Parses a MS Cabinet archive """
        dict.__init__(self)

        self.set_id: int = 0
        self._folder_data: List[bytearray] = []
        self._is_multi_folder: bool = False
        self._flattern: bool = flattern

        # load archive
        if buf:
            self.parse(buf)

    def __setitem__(self, key: str, val: CabFile) -> None:
        assert isinstance(key, str)
        assert isinstance(val, CabFile)
        val.filename = key
        dict.__setitem__(self, key, val)

    def _parse_cffile(self, buf: bytes, offset: int) -> int:
        """ Parse a CFFILE entry """
        fmt = "<I"  # uncompressed size
        fmt += "I"  # uncompressed offset of this file in the folder
        fmt += "H"  # index into the CFFOLDER area
        fmt += "H"  # date
        fmt += "H"  # time
        fmt += "H"  # attribs
        try:
            vals = struct.unpack_from(fmt, buf, offset)
        except struct.error as e:
            raise CorruptionError from e

        # parse filename
        offset += struct.calcsize(fmt)
        filename = ""
        for i in range(0, 255):
            if buf[offset + i] == 0x0:
                filename = buf[offset : offset + i].decode()
                break

        # add file
        f = CabFile()
        f._date_decode(vals[3])
        f._time_decode(vals[4])
        f._attr_decode(vals[5])
        try:
            f.buf = bytes(self._folder_data[vals[2]][vals[1] : vals[1] + vals[0]])
        except IndexError as e:
            raise CorruptionError( "Failed to get buf for {}: {}".format(filename, vals)) from e
        if len(f) != vals[0]:
            raise CorruptionError(
                "Corruption inside archive, %s is size %i but "
                "expected size %i" % (filename, len(f), vals[0])
            )
        if self._flattern:
            filename = ntpath.basename(filename)
        self[filename] = f

        # return offset to next entry
        return 16 + i + 1

    def _parse_cffolder(self, buf: bytes, idx: int, offset: int) -> None:
        """ Parse a CFFOLDER entry """
        fmt = "<I"  # offset to CFDATA
        fmt += "H"  # number of CFDATA blocks
        fmt += "H"  # compression type
        try:
            vals = struct.unpack_from(fmt, buf, offset)
        except struct.error as e:
            raise CorruptionError from e

        # no data blocks?
        if vals[1] == 0:
            raise CorruptionError("No CFDATA blocks")

        # no compression is supported
        if vals[2] == 0:
            is_zlib = False
        elif vals[2] == 1:
            is_zlib = True
        else:
            raise NotSupportedError("Compression type not supported")

        # not supported
        if is_zlib and self._is_multi_folder:
            raise NotSupportedError(
                "Compression unsupported in multi-folder archive: "
                "set FolderSizeThreshold=0 in the .ddf file"
            )

        # parse CDATA
        self._folder_data.append(bytearray())
        offset = vals[0]
        for _ in range(vals[1]):
            offset += self._parse_cfdata(buf, idx, offset, is_zlib)

    def _parse_cfdata(self, buf: bytes, idx: int, offset: int, is_zlib: bool) -> int:
        """ Parse a CFDATA entry """
        fmt = "<I"  # checksum
        fmt += "H"  # compressed bytes
        fmt += "H"  # uncompressed bytes
        try:
            vals = struct.unpack_from(fmt, buf, offset)
        except struct.error as e:
            raise CorruptionError from e
        if not is_zlib and vals[1] != vals[2]:
            raise CorruptionError("Mismatched data %i != %i" % (vals[1], vals[2]))
        hdr_sz = struct.calcsize(fmt)
        newbuf = buf[offset + hdr_sz : offset + hdr_sz + vals[1]]

        # decompress Zlib data after removing *another* header...
        if is_zlib:
            if newbuf[:2] != b"CK":
                raise CorruptionError(
                    "Compression header invalid {}".format(newbuf[:2].decode())
                )
            decompress = zlib.decompressobj(-zlib.MAX_WBITS)
            try:
                buf = decompress.decompress(newbuf[2:])
                buf += decompress.flush()
            except zlib.error as e:
                raise CorruptionError("Failed to decompress") from e
        else:
            buf = newbuf

        # check checksum
        if vals[0] != 0:
            checksum = _checksum_compute(newbuf)
            hdr = bytearray(struct.pack("<HH", len(newbuf), len(buf)))
            checksum = _checksum_compute(hdr, checksum)
            if checksum != vals[0]:
                raise CorruptionError("Invalid checksum", offset, vals[0], checksum)

        assert len(buf) == vals[2]
        self._folder_data[idx] += buf
        return vals[1] + hdr_sz

    def parse(self, buf: bytes) -> None:
        """ Parse .cab data """

        # read the file header
        fmt = "<4s"  # signature
        fmt += "xxxx"  # reserved1
        fmt += "I"  # size
        fmt += "xxxx"  # reserved2
        fmt += "I"  # offset to CFFILE
        fmt += "xxxx"  # reserved3
        fmt += "BB"  # version minor, major
        fmt += "H"  # no of CFFOLDERs
        fmt += "H"  # no of CFFILEs
        fmt += "H"  # flags
        fmt += "H"  # setID
        fmt += "H"  # cnt of cabs in set
        #        fmt += 'H'      # reserved cab size
        #        fmt += 'B'      # reserved folder size
        #        fmt += 'B'      # reserved block size
        #        fmt += 'B'      # per-cabinet reserved area
        try:
            vals = struct.unpack_from(fmt, buf, 0)
        except struct.error as e:
            raise CorruptionError from e

        # debugging
        if os.getenv("PYTHON_CABARCHIVE_DEBUG"):
            print("CFHEADER", vals)

        # check magic bytes
        if vals[0] != b"MSCF":
            raise NotSupportedError("Data is not application/vnd.ms-cab-compressed")

        # check size matches
        if vals[1] != len(buf):
            raise CorruptionError("Cab file internal size does not match data")

        # check version
        if vals[4] != 1 or vals[3] != 3:
            raise NotSupportedError(
                "Version {}.{} not supported".format(vals[4], vals[3])
            )

        # chained cabs not supported
        if vals[9] != 0:
            raise NotSupportedError("Chained cab file not supported")

        # verify we actually have data
        nr_files = vals[6]
        if nr_files == 0:
            raise CorruptionError("The cab file is empty")

        # verify we got complete data
        off_cffile = vals[2]
        if off_cffile > len(buf):
            raise CorruptionError("Cab file corrupt")

        # chained cabs not supported
        if vals[7] != 0:
            raise CorruptionError("Expected header flags to be cleared")

        # read this so we can do round-trip
        self.set_id = vals[8]

        # we don't support compressed folders in multi-folder archives
        if vals[5] > 1:
            self._is_multi_folder = True

        # parse CFFOLDER
        offset = struct.calcsize(fmt)
        for i in range(vals[5]):
            self._parse_cffolder(buf, i, offset)
            offset += struct.calcsize(FMT_CFFOLDER)

        # parse CFFILEs
        for i in range(0, nr_files):
            off_cffile += self._parse_cffile(buf, off_cffile)

    def find_file(self, glob: str) -> Optional[CabFile]:
        """ Gets a file from the archive using a glob """
        for fn in self:
            if fnmatch.fnmatch(fn, glob):
                return self[fn]
        return None

    def find_files(self, glob: str) -> List[CabFile]:
        """ Gets files from the archive using a glob """
        arr = []
        for fn in self:
            if fnmatch.fnmatch(fn, glob):
                arr.append(self[fn])
        return arr

    def save(self, compress: bool = False, sort: bool = True) -> bytes:
        """ Returns cabinet file data """

        # sort files before export
        cffiles: List[CabFile] = []
        if sort:
            for fn in sorted(self.keys()):
                cffiles.append(self[fn])
        else:
            cffiles.extend(self.values())

        # create linear CFDATA block
        cfdata_linear = bytearray()
        for f in cffiles:
            if f.buf:
                cfdata_linear += f.buf

        # _chunkify and compress with a fixed size
        chunks = _chunkify(cfdata_linear, 0x8000)
        if compress:
            chunks_zlib = []
            for chunk in chunks:
                compressobj = zlib.compressobj(9, zlib.DEFLATED, -zlib.MAX_WBITS)
                chunk_zlib = bytearray(b"CK")
                chunk_zlib += compressobj.compress(chunk)
                chunk_zlib += compressobj.flush()
                chunks_zlib.append(chunk_zlib)
        else:
            chunks_zlib = chunks

        # create header
        archive_size = struct.calcsize(FMT_CFHEADER)
        archive_size += struct.calcsize(FMT_CFFOLDER)
        for f in cffiles:
            if not f._filename_win32:
                continue
            archive_size += struct.calcsize(FMT_CFFILE) + len(f._filename_win32.encode()) + 1
        for chunk in chunks_zlib:
            archive_size += struct.calcsize(FMT_CFDATA) + len(chunk)
        offset = struct.calcsize(FMT_CFHEADER)
        offset += struct.calcsize(FMT_CFFOLDER)
        data = struct.pack(
            FMT_CFHEADER,
            b"MSCF",  # signature
            archive_size,  # complete size
            offset,  # offset to CFFILE
            3,
            1,  # ver minor major
            1,  # no of CFFOLDERs
            len(self),  # no of CFFILEs
            0,  # flags
            self.set_id,  # setID
            0,
        )  # cnt of cabs in set

        # create folder
        for f in cffiles:
            if not f._filename_win32:
                continue
            offset += struct.calcsize(FMT_CFFILE)
            offset += len(f._filename_win32.encode()) + 1
        data += struct.pack(
            FMT_CFFOLDER,
            offset,  # offset to CFDATA
            len(chunks),  # number of CFDATA blocks
            compress,
        )  # compression type

        # create each CFFILE
        index_into = 0
        for f in cffiles:
            if not f._filename_win32:
                continue
            data += struct.pack(
                FMT_CFFILE,
                len(f),  # uncompressed size
                index_into,  # uncompressed offset
                0,  # index into CFFOLDER
                f._date_encode(),  # date
                f._time_encode(),  # time
                f._attr_encode(),
            )  # attribs
            data += f._filename_win32.encode() + b"\0"
            index_into += len(f)

        # create each CFDATA
        for i in range(0, len(chunks)):
            chunk = chunks[i]
            chunk_zlib = chunks_zlib[i]

            # first do the 'checksum' on the data, then the partial
            # header. slightly crazy, but anyway
            checksum = _checksum_compute(chunk_zlib)
            hdr = bytearray(struct.pack("<HH", len(chunk_zlib), len(chunk)))
            checksum = _checksum_compute(hdr, checksum)
            data += struct.pack(
                FMT_CFDATA,
                checksum,  # checksum
                len(chunk_zlib),  # compressed bytes
                len(chunk),
            )  # uncompressed bytes
            data += chunk_zlib

        # return bytearray
        return data

    def __repr__(self) -> str:
        return "CabArchive({})".format([str(self[cabfile]) for cabfile in self])



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (C) 2015-2020 Richard Hughes <richard@hughsie.com>
#
# SPDX-License-Identifier: LGPL-2.1+


class CorruptionError(Exception):
    pass


class NotSupportedError(Exception):
    pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (C) 2015-2020 Richard Hughes <richard@hughsie.com>
#
# SPDX-License-Identifier: LGPL-2.1+
#
# pylint: disable=wrong-import-position

import os
import sys
import unittest
import datetime
import subprocess
import time
import hashlib

# allows us to run this from the project root
sys.path.append(os.path.realpath("."))

from cabarchive import CabArchive, CabFile, CorruptionError, NotSupportedError
from cabarchive.archive import _checksum_compute


def _check_archive(filename: str, expected_rc: int = 0) -> None:
    argv = ["cabextract", "--test", filename]
    rc = subprocess.call(argv)
    assert rc == expected_rc, "invalid return code: %r" % rc


def _check_range(data: bytes, expected: bytes) -> None:
    assert data
    assert expected
    failures = 0
    if len(data) != len(expected):
        print("different sizes, got %i expected %i" % (len(data), len(expected)))
        failures += 1
    for i in range(len(data)):
        if data[i] != expected[i]:
            print("@0x%02x got 0x%02x expected 0x%02x" % (i, data[i], expected[i]))
            failures += 1
            if failures > 10:
                print("More than 10 failures, giving up...")
                break
    assert failures == 0, "Data is not the same"


class TestInfParser(unittest.TestCase):
    def test_checksums(self):

        # test checksum function
        csum = _checksum_compute(b"hello123")
        self.assertEqual(csum, 0x5F5E5407)
        csum = _checksum_compute(b"hello")
        self.assertEqual(csum, 0x6C6C6507)

        # measure speed
        data = open("data/random.bin", "rb").read()
        start = time.time()
        csum = _checksum_compute(data)
        print("profile checksum: %fms" % ((time.time() - start) * 1000))

    def test_compressed(self):
        cabarchive = CabArchive()

        # make predictable
        dt_epoch = datetime.datetime.fromtimestamp(0, datetime.timezone.utc)
        cabarchive["README.txt"] = CabFile(b"foofoofoofoofoofoofoofoo", mtime=dt_epoch)
        cabarchive["firmware.bin"] = CabFile(b"barbarbarbarbarbarbarbar", mtime=dt_epoch)
        buf = cabarchive.save(compress=True)
        self.assertEqual(len(buf), 122)
        self.assertEqual(
            hashlib.sha1(buf).hexdigest(), "74e94703c403aa93b16d01b088eb52e3a9c73288"
        )

    def test_values(self):

        # parse junk
        arc = CabArchive()
        try:
            arc.parse(b"hello")
        except CorruptionError:
            pass

        # parse junk
        _check_archive("hello", expected_rc=1)

        # parse test files
        for fn in [
            "data/simple.cab",
            "data/compressed.cab",
            "data/utf8.cab",
            "data/large.cab",
            "data/large-compressed.cab",
        ]:
            arc = CabArchive()
            print("Parsing:", fn)
            with open(fn, "rb") as f:
                old = f.read()
                arc.parse(old)
            assert len(arc) == 1
            if arc.find_file("*.txt"):
                cff = list(arc.values())[0]
                self.assertEqual(cff.filename, "test.txt")
                assert cff.buf == b"test123", cff.buf
                assert len(cff.buf) == 7, "Expected 7, got %i" % len(cff.buf)
                assert cff.date.year == 2015
            elif arc.find_file("*.dat"):
                cff = list(arc.values())[0]
                assert cff.filename == "tést.dat", cff.filename
                assert cff.buf == "tést123".encode(), cff.buf
                assert len(cff.buf) == 8, "Expected 8, got %i" % len(cff.buf)
                assert cff.date.year == 2015
            else:
                cff = list(arc.values())[0]
                assert cff.filename == "random.bin", cff.filename
                assert len(cff.buf) == 0xFFFFF, "Expected 1 Mb, got %i" % len(cff.buf)
                assert (
                    hashlib.sha1(cff.buf).hexdigest()
                    == "8497fe89c41871e3cbd7955e13321e056dfbd170"
                ), "SHA hash incorrect"
                assert cff.date.year == 2015

            # make sure we don't modify on roundtrip
            compress = False
            if fn.find("compressed") != -1:
                compress = True
            new = arc.save(compress)
            _check_range(bytearray(new), bytearray(old))

        # create new archive
        arc = CabArchive()
        arc.set_id = 0x0622

        # first example
        cff = CabFile()
        cff.buf = b'#include <stdio.h>\r\n\r\nvoid main(void)\r\n{\r\n    printf("Hello, world!\\n");\r\n}\r\n'
        cff.date = datetime.date(1997, 3, 12)
        cff.time = datetime.time(11, 13, 52)
        cff.is_arch = True
        arc["hello.c"] = cff

        # second example
        cff = CabFile()
        cff.buf = b'#include <stdio.h>\r\n\r\nvoid main(void)\r\n{\r\n    printf("Welcome!\\n");\r\n}\r\n\r\n'
        cff.date = datetime.date(1997, 3, 12)
        cff.time = datetime.time(11, 15, 14)
        cff.is_arch = True
        arc["welcome.c"] = cff

        # verify
        data = arc.save(False)
        with open("/tmp/test.cab", "wb") as f:
            f.write(data)
        expected = (
            b"\x4D\x53\x43\x46\x00\x00\x00\x00\xFD\x00\x00\x00\x00\x00\x00\x00"
            b"\x2C\x00\x00\x00\x00\x00\x00\x00\x03\x01\x01\x00\x02\x00\x00\x00"
            b"\x22\x06\x00\x00\x5E\x00\x00\x00\x01\x00\x00\x00\x4D\x00\x00\x00"
            b"\x00\x00\x00\x00\x00\x00\x6C\x22\xBA\x59\x20\x00\x68\x65\x6C\x6C"
            b"\x6F\x2E\x63\x00\x4A\x00\x00\x00\x4D\x00\x00\x00\x00\x00\x6C\x22"
            b"\xE7\x59\x20\x00\x77\x65\x6C\x63\x6F\x6D\x65\x2E\x63\x00\xBD\x5A"
            b"\xA6\x30\x97\x00\x97\x00\x23\x69\x6E\x63\x6C\x75\x64\x65\x20\x3C"
            b"\x73\x74\x64\x69\x6F\x2E\x68\x3E\x0D\x0A\x0D\x0A\x76\x6F\x69\x64"
            b"\x20\x6D\x61\x69\x6E\x28\x76\x6F\x69\x64\x29\x0D\x0A\x7B\x0D\x0A"
            b"\x20\x20\x20\x20\x70\x72\x69\x6E\x74\x66\x28\x22\x48\x65\x6C\x6C"
            b"\x6F\x2C\x20\x77\x6F\x72\x6C\x64\x21\x5C\x6E\x22\x29\x3B\x0D\x0A"
            b"\x7D\x0D\x0A\x23\x69\x6E\x63\x6C\x75\x64\x65\x20\x3C\x73\x74\x64"
            b"\x69\x6F\x2E\x68\x3E\x0D\x0A\x0D\x0A\x76\x6F\x69\x64\x20\x6D\x61"
            b"\x69\x6E\x28\x76\x6F\x69\x64\x29\x0D\x0A\x7B\x0D\x0A\x20\x20\x20"
            b"\x20\x70\x72\x69\x6E\x74\x66\x28\x22\x57\x65\x6C\x63\x6F\x6D\x65"
            b"\x21\x5C\x6E\x22\x29\x3B\x0D\x0A\x7D\x0D\x0A\x0D\x0A"
        )
        _check_range(bytearray(data), bytearray(expected))

        # use cabextract to test validity
        argv = ["cabextract", "--test", "/tmp/test.cab"]
        rc = subprocess.call(argv)
        assert rc == 0

        # check we can parse what we just created
        arc = CabArchive()
        with open("/tmp/test.cab", "rb") as f:
            arc.parse(f.read())

        # add an extra file
        arc["test.inf"] = CabFile(b"$CHICAGO$")

        # save with compression
        with open("/tmp/test.cab", "wb") as f:
            f.write(arc.save(True))

        # use cabextract to test validity
        _check_archive("/tmp/test.cab")

        # open a folder with multiple folders
        for fn in ["data/multi-folder.cab", "data/ddf-fixed.cab"]:
            arc = CabArchive()
            print("Parsing:", fn)
            old = open(fn, "rb").read()
            arc.parse(old)
            assert len(arc) == 2, len(arc)
            cff = arc.find_file("*.txt")
            assert cff.buf == b"test123", cff.buf

        # parse junk
        arc = CabArchive()
        try:
            with open("data/multi-folder-compressed.cab", "rb") as f:
                arc.parse(f.read())
        except NotSupportedError as _:
            pass


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (C) 2015-2020 Richard Hughes <richard@hughsie.com>
#
# SPDX-License-Identifier: LGPL-2.1+

import datetime

from typing import Optional


def _is_ascii(text: str) -> bool:
    """ Check if a string is ASCII only """
    if not text:
        return False
    return all(ord(c) < 128 for c in text)


class CabFile:

    """An object representing a file in a Cab archive """

    def __init__(
        self,
        buf: Optional[bytes] = None,
        filename: Optional[str] = None,
        mtime: Optional[datetime.datetime] = None,
    ):
        self.filename = filename
        self.buf = buf
        self.date: Optional[datetime.date]
        self.time: Optional[datetime.time]
        if mtime:
            self.date = mtime.date()
            self.time = mtime.time()
        else:
            self.date = datetime.date.today()
            self.time = datetime.datetime.now().time()
        self.is_readonly = False  # file is read-only
        self.is_hidden = False  # file is hidden
        self.is_system = False  # file is a system file
        self.is_arch = False  # file modified since last backup
        self.is_exec = False  # file is executable

    def __len__(self) -> int:
        if not self.buf:
            return 0
        return len(self.buf)

    @property
    def filename(self) -> Optional[str]:
        return self._filename

    @filename.setter
    def filename(self, filename: str) -> None:
        self.is_name_utf8 = not _is_ascii(filename)
        self._filename = filename

    @property
    def _filename_win32(self) -> Optional[str]:
        return self._filename.replace("/", "\\")

    def _attr_encode(self) -> int:
        """ Get attributes on the file """
        attr = 0x00
        if self.is_readonly:
            attr += 0x01
        if self.is_hidden:
            attr += 0x02
        if self.is_system:
            attr += 0x04
        if self.is_arch:
            attr += 0x20
        if self.is_exec:
            attr += 0x40
        if self.is_name_utf8:
            attr += 0x80
        return attr

    def _attr_decode(self, attr: int) -> None:
        """ Set attributes on the file """
        self.is_readonly = bool(attr & 0x01)
        self.is_hidden = bool(attr & 0x02)
        self.is_system = bool(attr & 0x04)
        self.is_arch = bool(attr & 0x20)
        self.is_exec = bool(attr & 0x40)
        self.is_name_utf8 = bool(attr & 0x80)

    def _date_decode(self, val: int) -> None:
        """ Decode the MSCAB 32-bit date format """
        try:
            self.date = datetime.date(
                1980 + ((val & 0xFE00) >> 9), (val & 0x01E0) >> 5, val & 0x001F
            )
        except ValueError as _:
            self.date = None

    def _time_decode(self, val: int) -> None:
        """ Decode the MSCAB 32-bit time format """
        try:
            self.time = datetime.time(
                (val & 0xF800) >> 11, (val & 0x07E0) >> 5, (val & 0x001F) * 2
            )
        except ValueError as _:
            self.time = None

    def _date_encode(self) -> int:
        """ Encode the MSCAB 32-bit date format """
        if not self.date or self.date.year < 1980:
            return 0
        return ((self.date.year - 1980) << 9) + (self.date.month << 5) + self.date.day

    def _time_encode(self) -> int:
        """ Encode the MSCAB 32-bit time format """
        if not self.time:
            return 0
        return (
            (self.time.hour << 11) + (self.time.minute << 5) + int(self.time.second / 2)
        )

    def __repr__(self) -> str:
        return "CabFile({}:{:x})".format(self.filename, len(self))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (C) 2015-2020 Richard Hughes <richard@hughsie.com>
#
# SPDX-License-Identifier: LGPL-2.1+

from cabarchive.file import CabFile
from cabarchive.archive import CabArchive
from cabarchive.errors import CorruptionError, NotSupportedError



#!/usr/bin/env python

from setuptools import setup

# note: this is a repeat of the README, to evolve, good enough for now.
long_desc = """
If you want to parse Microsoft Cabinet files in Python you probably
should just install gcab, and use the GObjectIntrospection bindings
for that. GCab is a much better library than this and handles many more
kinds of archive.

If GCab is not available to you (e.g. you're trying to run in an
OpenShift instance on RHEL 6.2), this project might be somewhat useful.

Contributors welcome, either adding new functionality or fixing bugs.

See also: https://msdn.microsoft.com/en-us/library/bb417343.aspx
"""

setup(
    name="cabarchive",
    version="0.2.1",
    license="LGPL-2.1-or-later",
    description="A pure-python library for creating and extracting cab files",
    long_description=long_desc,
    author="Richard Hughes",
    author_email="richard@hughsie.com",
    url="https://github.com/hughsie/python-cabarchive",
    packages=[
        "cabarchive",
    ],
    include_package_data=True,
    zip_safe=False,
    classifiers=[
        # complete classifier list: http://pypi.python.org/pypi?%3Aaction=list_classifiers
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: GNU Lesser General Public License v2 or later (LGPLv2+)",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2.7",
        "Topic :: Utilities",
        "Topic :: System :: Archiving",
    ],
    keywords=["cabextract", "cab", "archive", "extract"],
    package_data={
        "cabarchive": [
            "py.typed",
            "archive.pyi",
            "errors.pyi",
            "file.pyi",
            "__init__.pyi",
        ]
    },
)
